==> clazy_v1.1-src.zip: clazy-1.1/src/checks/level2/container-inside-loop.cpp <==
/*
   This file is part of the clazy static checker.

  Copyright (C) 2016 Sergio Martins <smartins@kde.org>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; see the file COPYING.LIB.  If not, write to
  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  Boston, MA 02110-1301, USA.
*/

#include "container-inside-loop.h"
#include "Utils.h"
#include "checkmanager.h"
#include "StringUtils.h"
#include "LoopUtils.h"
#include "StmtBodyRange.h"

#include <clang/AST/ParentMap.h>
#include <clang/AST/AST.h>
#include <clang/Lex/Lexer.h>

using namespace clang;
using namespace std;


ContainerInsideLoop::ContainerInsideLoop(const std::string &name, const clang::CompilerInstance &ci)
    : CheckBase(name, ci)
{
}

void ContainerInsideLoop::VisitStmt(clang::Stmt *stmt)
{
    auto *ctorExpr = dyn_cast<CXXConstructExpr>(stmt);
    if (!ctorExpr)
        return;

    CXXConstructorDecl *ctor = ctorExpr->getConstructor();
    if (!ctor || !clazy_std::equalsAny(StringUtils::classNameFor(ctor), { "QVector", "std::vector", "QList" }))
        return;

    DeclStmt *declStm = dyn_cast_or_null<DeclStmt>(m_parentMap->getParent(stmt));
    if (!declStm || !declStm->isSingleDecl())
        return;

    Stmt *loopStmt = LoopUtils::isInLoop(m_parentMap, stmt);
    if (!loopStmt)
        return;

    auto *varDecl = dyn_cast<VarDecl>(declStm->getSingleDecl());
    if (!varDecl || Utils::isInitializedExternally(varDecl))
        return;

    if (Utils::isPassedToFunction(StmtBodyRange(loopStmt), varDecl, true))
        return;

    emitWarning(stmt->getLocStart(), "container inside loop causes unneeded allocations");
}


REGISTER_CHECK_WITH_FLAGS("container-inside-loop", ContainerInsideLoop, CheckLevel2)

==> clazy_v1.2-src.zip: clazy-1.2/src/checks/level2/container-inside-loop.cpp <==
/*
   This file is part of the clazy static checker.

  Copyright (C) 2016 Sergio Martins <smartins@kde.org>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; see the file COPYING.LIB.  If not, write to
  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  Boston, MA 02110-1301, USA.
*/

#include "container-inside-loop.h"
#include "Utils.h"
#include "checkmanager.h"
#include "StringUtils.h"
#include "LoopUtils.h"
#include "StmtBodyRange.h"

#include <clang/AST/ParentMap.h>
#include <clang/AST/AST.h>
#include <clang/Lex/Lexer.h>

using namespace clang;
using namespace std;


ContainerInsideLoop::ContainerInsideLoop(const std::string &name, ClazyContext *context)
    : CheckBase(name, context)
{
}

void ContainerInsideLoop::VisitStmt(clang::Stmt *stmt)
{
    auto *ctorExpr = dyn_cast<CXXConstructExpr>(stmt);
    if (!ctorExpr)
        return;

    CXXConstructorDecl *ctor = ctorExpr->getConstructor();
    if (!ctor || !clazy_std::equalsAny(StringUtils::classNameFor(ctor), { "QVector", "std::vector", "QList" }))
        return;

    DeclStmt *declStm = dyn_cast_or_null<DeclStmt>(m_context->parentMap->getParent(stmt));
    if (!declStm || !declStm->isSingleDecl())
        return;

    Stmt *loopStmt = LoopUtils::isInLoop(m_context->parentMap, stmt);
    if (!loopStmt)
        return;

    auto *varDecl = dyn_cast<VarDecl>(declStm->getSingleDecl());
    if (!varDecl || Utils::isInitializedExternally(varDecl))
        return;

    if (Utils::isPassedToFunction(StmtBodyRange(loopStmt), varDecl, true))
        return;

    emitWarning(stmt->getLocStart(), "container inside loop causes unneeded allocations");
}


REGISTER_CHECK("container-inside-loop", ContainerInsideLoop, CheckLevel2)
